#
# spec file for package ghc-effin
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%global pkg_name effin
Name:           ghc-%{pkg_name}
Version:        0.3.0.3
Release:        4.2
Summary:        A Typeable-free implementation of extensible effects
License:        BSD-3-Clause
Group:          Development/Libraries/Haskell
URL:            https://hackage.haskell.org/package/%{pkg_name}
Source0:        https://hackage.haskell.org/package/%{pkg_name}-%{version}/%{pkg_name}-%{version}.tar.gz
BuildRequires:  ghc-Cabal-devel
BuildRequires:  ghc-mtl-devel
BuildRequires:  ghc-rpm-macros

%description
This package implements extensible effects, an alternative to monad
transformers. The original paper can be found at
<http://okmij.org/ftp/Haskell/extensible/exteff.pdf>. The main differences
between this library and the one described in the paper are that this library
does not use the Typeable type class, does not require that effects implement
the Functor type class, and has a simpler API for handling effects.

For example, the following code implements a handler for exceptions:

> newtype Exception e = Throw e > > runException :: Effect (Exception e :+ es)
a -> Effect es (Either e a) > runException = eliminate > (x -> return (Right
x)) > ((Throw e) k -> return (Left e))

Compare this to the corresponding code in extensible-effects
(<http://hackage.haskell.org/package/extensible-effects>):

> runExc :: Typeable e => Eff (Exc e :> r) a -> Eff r (Either e a) > runExc =
loop . admin > where > loop (Val x) = return (Right x) > loop (E u) =
handleRelay u loop ((Exc e) -> return (Left e))

In particular, effect implementors are not required to do any recursion,
thereby making effect handlers more composeable.

%package devel
Summary:        Haskell %{pkg_name} library development files
Group:          Development/Libraries/Haskell
Requires:       %{name} = %{version}-%{release}
Requires:       ghc-compiler = %{ghc_version}
Requires(post): ghc-compiler = %{ghc_version}
Requires(postun): ghc-compiler = %{ghc_version}

%description devel
This package provides the Haskell %{pkg_name} library development files.

%prep
%setup -q -n %{pkg_name}-%{version}

%build
%ghc_lib_build

%install
%ghc_lib_install

%post devel
%ghc_pkg_recache

%postun devel
%ghc_pkg_recache

%files -f %{name}.files
%license LICENSE

%files devel -f %{name}-devel.files

%changelog
* Wed Mar 14 2018 psimons@suse.com
- Update effin to version 0.3.0.3.
  A more detailed change log is not available.
